/*****************************************************************************
 *                                                                           *
 *                Licensed Materials - Property of IBM Corp.                 *
 *                  5648-C60 (c) Copyright IBM Corp. 2000                    *
 *                                                                           *
 *       MQJMS Standalone Application Server Facilities (ASF) Samples        *
 *                                                                           *
 *                                                                           *
 *  FILENAME:     ASFClient2.java                                            *
 *                                                                           *
 *  DESCRIPTION:  Second in a series of sample applications which can be     *
 *                used to exercise:                                          *
 *                    - the Application Server Facilities in MQJMS           *
 *                    - the sample standalone application server environ-    *
 *                      ment supplied                                        *
 *                                                                           *
 *                Further information about this class and its usage can be  *
 *                found in the 'Websphere MQ Using Java' documentation.      *
 *                                                                           *
 *  USAGE:        java ASFClient2 [options]                                  *
 *                                                                           *
 *                This application accepts the same command-line options     *
 *                as the ASFClient1 application. See the Websphere MQ Using  *
 *                Java documentation for more details.                       *
 *                                                                           *
 *  HISTORY:      010131 - First released as part of SupportPac MA88 v2      *
 *                                                                           *
 *****************************************************************************/

import javax.jms.*;
import javax.naming.*;
import java.util.Hashtable;
import java.io.IOException;

public class ASFClient2 {
   // These are the defaults for the parameters which can be over-ridden
   // on the command line

   // Will we use JNDI for administered object creation?
   private static boolean useJNDI   = true;
   // JNDI parameters
   private static String  qcfLookup = "cn=qcf";
   private static String  qLookup   = "cn=q";
   private static String  icf       = "com.sun.jndi.ldap.LdapCtxFactory";
   private static String  url       = "ldap://localhost/o=ibm,c=us";
   // Non-JNDI parameters
   private static String  qMgrName  = "";
   private static String  qName     = "SYSTEM.DEFAULT.LOCAL.QUEUE";
   // Other parameters
   private static int     poolSize  = 5;
   private static int     batchSize = 10;

   /**************************************************************************
    *  main - Application entry point                                        *
    **************************************************************************/
   public static void main( String[] args ) throws JMSException, IOException
   {
      // Parse the command line
      parseArguments(args);

      // Obtain the two administered objects we need
      Object[] adminObjects = new Object[2];
      obtainAdminObjects( adminObjects );
      QueueConnectionFactory qcf = (QueueConnectionFactory)adminObjects[0];
      Queue q = (Queue)adminObjects[1];

      // Obtain a QueueConnection
      QueueConnection conn = qcf.createQueueConnection();

      // Create a MessageListenerFactory for each ServerSessionPool
      CountingMessageListenerFactory cmlf = new CountingMessageListenerFactory();
      cmlf.setName("CountingFactory");
      LoggingMessageListenerFactory lmlf = new LoggingMessageListenerFactory(
                                                          "LoggingFactory",
                                                          "./ASFClient2.log" );

      // Create two ServerSessionPools using our simple MyServerSessionPool
      // implementation
      MyServerSessionPool normalSSP =
           new MyServerSessionPool( conn,         // The JMS Connection to use
                                    poolSize,           // The pool's capacity
                                    Session.AUTO_ACKNOWLEDGE,  // The ack mode
                                    cmlf );   // MessageListenerFactory to use

      MyServerSessionPool highSSP =
           new MyServerSessionPool( conn,         // The JMS Connection to use
                                    poolSize,           // The pool's capacity
                                    Session.AUTO_ACKNOWLEDGE,  // The ack mode
                                    lmlf );   // MessageListenerFactory to use

      // Finally, create the two ConnectionConsumers and then start things
      // going by starting the connection
      System.out.print("Creating ConnectionConsumer(1)... ");
      conn.createConnectionConsumer( q,     // The Destination to consume from
                                     "value>75",    // 'high' message selector
                                     highSSP,  // The ServerSessionPool to use
                                     batchSize );// Maximum message batch size
      System.out.print("Done.\nCreating ConnectionConsumer(2)... ");
      conn.createConnectionConsumer( q,     // The Destination to consume from
                                     "value<=75", // 'normal' message selector
                                     normalSSP,// The ServerSessionPool to use
                                     batchSize );// Maximum message batch size
      System.out.print("Done.\nStarting Connection... ");
      conn.start();
      System.out.println("Done.");

      // Now, every ten seconds for a minute, get the CountingMessageListener
      // factory to print out its stats; the LoggingMessageListener's data
      // will get written to file
      for( int i=0; i<6; i++ ) {
         try {
            // Application thread sleeps for 10 seconds
            Thread.sleep(10000);
            System.out.println("After "+(i+1)*10+" seconds:");
         } catch(InterruptedException ie) {
            // Ignore
         }
         cmlf.printStats();
      }

      // After this minute of activity, close the Connection and the
      // ServerSessionPools and quit
      conn.close();
      normalSSP.close();
      highSSP.close();
   }

   /**************************************************************************
    *  parseArguments - Parse the given argument array, updating member      *
    *                   variables as we go. Terminates the application if    *
    *                   an error is encountered.                             *
    **************************************************************************/
   private static void parseArguments( String[] args ) {
      try {
         for( int i=0; i<args.length; i++ ) {
            String arg = args[i].toUpperCase();

            if( arg.equals("-NOJNDI"))         useJNDI   = false;
            else if( arg.equals("-URL"))       url       = args[++i];
            else if( arg.equals("-ICF"))       icf       = args[++i];
            else if( arg.equals("-QCFLOOKUP")) qcfLookup = args[++i];
            else if( arg.equals("-QLOOKUP"))   qLookup   = args[++i];
            else if( arg.equals("-Q"))         qName     = args[++i];
            else if( arg.equals("-QMGR"))      qMgrName  = args[++i];
            else if( arg.equals("-POOLSIZE"))  poolSize  = Integer.parseInt(
                                                                  args[++i]);
            else if( arg.equals("-BATCHSIZE")) batchSize = Integer.parseInt(
                                                                  args[++i]);
            else
               throw new Exception("Unrecognised argument ("+arg+")");
         }
      } catch( Exception e ) {
         System.out.println("Error parsing arguments: "+e+"\nQuitting.");
         System.exit(-1);
      }
   }

   /**************************************************************************
    *  obtainAdminObjects - Obtains the two administered objects we need and *
    *                       places references to them in the given array.    *
    *                       The method used (JNDI versus explicit construct- *
    *                       ion) is determined by the value of 'useJNDI'.    *
    *                                                                        *
    *                       Terminates the application if any problems are   *
    *                       encountered.                                     *
    **************************************************************************/
   private static void obtainAdminObjects( Object[] outArray ) {
      try {
         if( useJNDI ) {
            // Set up JNDI environment parameters
            Hashtable env = new Hashtable();
            env.put( Context.PROVIDER_URL, url );
            env.put( Context.INITIAL_CONTEXT_FACTORY, icf );

            // Access the JNDI namespace
            Context ctx = new InitialContext(env);

            // Obtain the objects
            outArray[0] = ctx.lookup(qcfLookup);
            outArray[1] = ctx.lookup(qLookup);

         } else {
            // Create the objects explicitly - we need to refer to the
            // MQJMS-specific classes now
            com.ibm.mq.jms.MQQueueConnectionFactory qcf =
            new com.ibm.mq.jms.MQQueueConnectionFactory();
            qcf.setQueueManager(qMgrName);

            com.ibm.mq.jms.MQQueue q = new com.ibm.mq.jms.MQQueue();
            q.setBaseQueueName(qName);
            q.setBaseQueueManagerName(qMgrName);

            outArray[0] = qcf;
            outArray[1] = q;
         }
      } catch( Exception e ) {
         System.out.println("Error obtaining admin objects: "+e+"\nQuitting.");
         System.exit(-1);
      }
   }
}

